"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const anypoint_cli_command_1 = require("anypoint-cli-command");
const uris_1 = require("../../../helpers/uris");
const utils_1 = require("../../../helpers/utils");
const _ = require("lodash");
const utils_2 = require("anypoint-cli-command/lib/utils");
const FormData = require('form-data');
const fs = require('fs-extra');
class Copy extends anypoint_cli_command_1.AuthenticationBaseCommand {
    async run() {
        const { args, flags } = await this.parse(Copy);
        await this.cloudhubApplicationCopy(args.source, args.target, flags.property);
    }
    async cloudhubApplicationCopy(source, target, properties) {
        var _a, _b, _c, _d;
        let targetSession;
        let noTargetAppChecked = false;
        try {
            const data = {};
            const sourceSession = (0, utils_1.parseAppCreds)(source);
            sourceSession.envId = await (0, utils_2.getEnvironmentByName)(sourceSession.environment, sourceSession.organization);
            targetSession = (0, utils_1.parseAppCreds)(target);
            targetSession.envId = await (0, utils_2.getEnvironmentByName)(targetSession.environment, targetSession.organization);
            await this.checkNoTargetApp(targetSession);
            noTargetAppChecked = true;
            this.log('Loading source application');
            const sourceApp = await (0, utils_1.getSourceApp)(this, sourceSession);
            data.sourceApp = sourceApp.data;
            if (properties) {
                this.updateProperties(properties, data.sourceApp.properties);
            }
            this.log('Loading source application zip');
            const sourceZip = await (0, utils_1.downloadSourceZip)(this, sourceSession, data.sourceApp);
            data.zipPath = sourceZip;
            this.log('Copying source application to target');
            const appData = (0, utils_1.cleanAppData)(sourceApp.data, targetSession);
            const fd = new FormData();
            fd.append('appInfoJson', JSON.stringify(appData));
            fd.append('file', fs.createReadStream(data.zipPath));
            await this.req.post(uris_1.cloudAppsV2, fd, { headers: {
                    ...fd.getHeaders(),
                    'x-from-service': 'anypoint-cli',
                    'x-from-service-version': 'v2',
                    'X-ANYPNT-ENV-ID': targetSession.envId.id,
                    'X-ANYPNT-ORG-ID': targetSession.organization
                } });
            if (fs.existsSync(data.zipPath)) {
                fs.remove(data.zipPath);
            }
            await this.allocateTargetStaticIPs(this, targetSession, data.sourceApp);
            const sourceRunning = _.includes(['STARTED', 'DEPLOYING'], data.sourceApp.status);
            const toStatus = sourceRunning ? 'start' : 'stop';
            this.log(`Updating target application state: ${toStatus}`);
            await this.updateApplicationStateExt(targetSession, targetSession.app, toStatus);
        }
        catch (error) {
            try {
                if (targetSession && noTargetAppChecked) {
                    await this.req.delete((0, uris_1.cloudApp)(targetSession.app), { headers: { 'X-ANYPNT-ENV-ID': targetSession.envId.id,
                            'X-ANYPNT-ORG-ID': targetSession.organization } });
                }
            }
            catch {
                const errorMsg = ((_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? error.response.data.message : error;
                this.error(errorMsg);
            }
            const errorMsg = ((_d = (_c = error.response) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.message) ? error.response.data.message : error;
            this.error(errorMsg);
        }
    }
    async checkNoTargetApp(targetSess) {
        let targetApp;
        try {
            targetApp = await this.req.get((0, uris_1.cloudAppV2)(targetSess.app), { headers: { 'X-ANYPNT-ENV-ID': targetSess.envId.id,
                    'X-ANYPNT-ORG-ID': targetSess.organization } });
        }
        catch { }
        if (targetApp) {
            this.error(`Application named ${targetSess.app} already exists`);
        }
    }
    updateProperties(argProp, appProp) {
        const parsedProp = {};
        this.parseProperties(argProp, parsedProp);
        for (const prop of Object.keys(parsedProp)) {
            appProp[prop] = parsedProp[prop];
        }
    }
    parseProperties(propStrings, appProps) {
        appProps = appProps || {};
        if (!_.isArray(propStrings)) {
            propStrings = [propStrings];
        }
        _.each(propStrings, p => {
            const parts = this.splitPropertyArg(p);
            appProps[parts[0]] = parts[1];
        });
        return appProps;
    }
    splitPropertyArg(p) {
        const secureString = 'secure::';
        let propertyPrefix = '';
        p = p.replace(/\\=/gi, '=').replace(/\\:/gi, ':');
        if (p.startsWith(secureString)) {
            p = p.replace(/^secure::/gi, '');
            propertyPrefix = secureString;
        }
        const firstDelimiterIndex = [
            p.indexOf('='),
            p.indexOf(':'),
            // eslint-disable-next-line unicorn/no-array-reduce
        ].reduce((minIndex, currentIndex) => {
            if (currentIndex > -1 && currentIndex < minIndex) {
                return currentIndex;
            }
            return minIndex;
        }, Number.POSITIVE_INFINITY);
        if (firstDelimiterIndex < 0 || firstDelimiterIndex === Number.POSITIVE_INFINITY) {
            throw new Error('Invalid property format, expected name:value but was ' + p);
        }
        return [propertyPrefix + p.slice(0, Math.max(0, firstDelimiterIndex)), p.slice(Math.max(0, firstDelimiterIndex + 1))];
    }
    async allocateTargetStaticIPs(self, targetSess, sourceApp) {
        // Source app doesn't use static IPs or has no IPs allocated
        const hasIps = sourceApp.ipAddresses && sourceApp.ipAddresses.length > 0;
        if (!sourceApp.staticIPsEnabled || !hasIps) {
            return Promise.resolve();
        }
        self.log('Allocating static IPs for target application');
        const sourceRegions = _.map(sourceApp.ipAddresses, 'regionId');
        const proms = _.map(sourceRegions, function (region) {
            return self.req.post((0, uris_1.cloudAppStaticIPs)(targetSess.app), { region: region }, { headers: { 'X-ANYPNT-ENV-ID': targetSess.envId.id,
                    'X-ANYPNT-ORG-ID': targetSess.organization } });
        });
        return Promise.all(proms);
    }
    async updateApplicationStateExt(session, name, state) {
        var _a;
        try {
            const req = {
                url: (0, uris_1.cloudAppStatus)(name),
                body: { status: state },
                headers: { 'Content-Type': 'application/json',
                    'X-ANYPNT-ENV-ID': session.envId.id,
                    'X-ANYPNT-ORG-ID': session.organization },
            };
            await this.req.post(req.url, req.body, { headers: req.headers });
            this.log('OK');
        }
        catch (error) {
            if (((_a = error.response) === null || _a === void 0 ? void 0 : _a.status) === 304) {
                this.log('Application already in this state');
            }
            else {
                throw error;
            }
        }
    }
}
exports.default = Copy;
Copy.description = 'Copy cloud app from source to target. "source" and "target" args format: [<org>[:<env>/]]<app>';
Copy.args = [{ name: 'source', required: true },
    { name: 'target', required: true }];
Copy.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    property: core_1.Flags.string({ description: 'Set a property (name:value). Can be specified multiple times', multiple: true, required: false }),
};
