"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleCommonCreate = exports.getAppStatus = exports.showApplication = exports.parseRef = exports.buildHttpForUpdate = exports.handleCommonUpdate = exports.splitPropertyArg = exports.parseProperties = exports.parseFile = void 0;
const fs = require("fs");
const _ = require("lodash");
const util_1 = require("util");
const moment = require("moment");
const utils_1 = require("./helpers/utils");
const MULE_AGENT_PROP_SERVICE = 'mule.agent.application.properties.service';
const parseFile = (filePath, parserFunc) => {
    let result = {};
    const contents = fs.readFileSync(filePath, 'utf8');
    const lineEnding = /\r?\n|\r(?!\n)/;
    let lines = contents.split(lineEnding);
    lines = _.filter(lines, function (line) {
        return line && line.length > 0;
    });
    if (lines.length > 0) {
        result = parserFunc(lines);
    }
    return result;
};
exports.parseFile = parseFile;
const parseProperties = (propStrings, appProps) => {
    appProps = appProps || {};
    if (!_.isArray(propStrings)) {
        propStrings = [propStrings];
    }
    _.each(propStrings, function (p) {
        const parts = (0, exports.splitPropertyArg)(p);
        appProps[parts[0]] = parts[1];
    });
    return appProps;
};
exports.parseProperties = parseProperties;
const splitPropertyArg = (p) => {
    const secureString = 'secure::';
    let propertyPrefix = '';
    p = p.replace(/\\=/gi, '=').replace(/\\:/gi, ':');
    if (p.startsWith(secureString)) {
        p = p.replace(/^secure::/gi, '');
        propertyPrefix = secureString;
    }
    let firstDelimiterIndex;
    if (p.includes('=') && p.includes(':')) {
        firstDelimiterIndex = p.indexOf('=') < p.indexOf(':') ? p.indexOf('=') : p.indexOf(':');
    }
    else {
        firstDelimiterIndex = p.includes(':') ? p.indexOf(':') : p.indexOf('=');
    }
    if (firstDelimiterIndex < 0 || firstDelimiterIndex === Number.POSITIVE_INFINITY) {
        throw new Error('Invalid property format, expected name:value but was ' + p);
    }
    return [propertyPrefix + p.slice(0, Math.max(0, firstDelimiterIndex)), p.slice(Math.max(0, firstDelimiterIndex + 1))];
};
exports.splitPropertyArg = splitPropertyArg;
const handleCommonUpdate = (args, flags, orgId) => {
    const appData = {};
    if (flags.replicaSize) {
        _.set(appData, 'vCores', flags.replicaSize);
    }
    if (flags.assetVersion || flags.artifactId) {
        _.set(appData, 'ref', (0, exports.parseRef)(args, flags, orgId));
    }
    if (_.has(flags, 'objectStoreV2')) {
        _.set(appData, 'integrations', {
            services: {
                objectStoreV2: { enabled: flags.objectStoreV2 },
            },
        });
    }
    // deploy configuration
    const values = {};
    values[MULE_AGENT_PROP_SERVICE] = {};
    if (_.has(flags, 'propertiesFile')) {
        values[MULE_AGENT_PROP_SERVICE].properties =
            (0, exports.parseFile)(flags.propertiesFile, exports.parseProperties);
    }
    else if (_.has(flags, 'property')) {
        values[MULE_AGENT_PROP_SERVICE].properties =
            (0, exports.parseProperties)(flags.property, {});
    }
    if (_.has(flags, 'scopeLoggingConfig')) {
        const scopeConfig = [];
        for (const x of flags.scopeLoggingConfig.split(',')) {
            const z = x.split(':');
            scopeConfig.push({ scope: z[0], logLevel: z[1] });
        }
        values['mule.agent.logging.service'] = {
            scopeLoggingConfigurations: scopeConfig,
        };
    }
    _.set(appData, 'configuration', values);
    // target
    const target = {};
    const deploymentSettings = {};
    if (flags.replicas && flags.replicas > 0) {
        target.replicas = flags.replicas;
    }
    if (_.has(flags, 'clustered')) {
        deploymentSettings.clustered = flags.clustered;
    }
    if (_.has(flags, 'disableAmLogForwarding')) {
        deploymentSettings.disableAmLogForwarding =
            flags.disableAmLogForwarding;
    }
    if (flags.runtimeVersion) {
        deploymentSettings.runtimeVersion = flags.runtimeVersion;
    }
    if (flags.updateStrategy) {
        deploymentSettings.updateStrategy = flags.updateStrategy;
    }
    // http
    const inbound = (0, exports.buildHttpForUpdate)(flags);
    deploymentSettings.http = { inbound };
    target.deploymentSettings = deploymentSettings;
    return { application: appData, target: target };
};
exports.handleCommonUpdate = handleCommonUpdate;
const buildHttpForUpdate = (flags) => {
    const inbound = {};
    if (_.has(flags, 'publicEndpoints')) {
        inbound.publicUrl = flags.publicEndpoints;
    }
    if (_.has(flags, 'pathRewrite')) {
        inbound.pathRewrite = flags.pathRewrite;
    }
    if (_.has(flags, 'lastMileSecurity')) {
        inbound.lastMileSecurity = flags.lastMileSecurity;
    }
    if (_.has(flags, 'forwardSslSession')) {
        inbound.forwardSslSession = flags.forwardSslSession;
    }
    return inbound;
};
exports.buildHttpForUpdate = buildHttpForUpdate;
const parseRef = (args, flags, orgId) => {
    const groupId = flags.groupId ?
        flags.groupId : orgId;
    return {
        artifactId: args.artifactId || flags.artifactId,
        groupId: groupId,
        packaging: 'jar',
        version: flags.assetVersion ? flags.assetVersion : false,
    };
};
exports.parseRef = parseRef;
const showApplication = (self, app) => {
    const table = [];
    const dataTable = [];
    table.push(['Name: ' + app.name], ['App Id: ' + app.id], ['Status: ' + (0, exports.getAppStatus)(app)], ['Updated: ' + moment.utc(app.lastModifiedDate).fromNow()], ['Runtime: ' + app.target.deploymentSettings.runtimeVersion.split(':')[0]], ['Target: ' + app.target.targetId], ['Artifact Id :' + app.application.ref.artifactId], ['Replicas: ' + app.target.replicas], ['vCores per replica: ' + app.application.vCores], ['Update Strategy: ' + app.target.deploymentSettings.updateStrategy]);
    const spc = (0, utils_1.getSpecs)(app);
    table.push(['Specs: ' + spc.specs.join(',\n')]);
    if (app.target.deploymentSettings.http.inbound &&
        app.target.deploymentSettings.http.inbound.publicUrl) {
        table.push([
            'Public Endpoint: ' + app.target.deploymentSettings.http.inbound.publicUrl,
        ]);
    }
    const data = app.application.configuration['mule.agent.application.properties.service'].properties;
    const arr = [];
    for (const key in data) {
        if (key) {
            const value = data[key];
            arr.push((0, util_1.format)('%s : %s', key, value));
        }
    }
    table.push([
        'Properties: ' + arr.join(',\n'),
    ]);
    if (spc.errs.length > 0) {
        table.push([
            'Desired config not applied, 1 or more replicas :' + spc.errs.join(',\n'),
        ]);
    }
    self.show(table, dataTable);
};
exports.showApplication = showApplication;
const getAppStatus = (app) => {
    if (app.deploymentUpdateStatus === 'DEPLOY_FAILED' &&
        app.status === 'STARTED') {
        return 'STARTED\n(Update failed)';
    }
    return app.status;
};
exports.getAppStatus = getAppStatus;
const handleCommonCreate = (args, flags, orgId) => {
    const appData = {};
    _.set(appData, 'desiredState', 'STARTED');
    _.set(appData, 'vCores', flags.replicaSize ? flags.replicaSize : '0.1');
    _.set(appData, 'assets', []);
    _.set(appData, 'ref', (0, exports.parseRef)(args, flags, orgId));
    _.set(appData, 'configuration', getDeployConfig(args, flags));
    _.set(appData, 'integrations', {
        services: {
            objectStoreV2: {
                enabled: flags.objectStoreV2 ? flags.objectStoreV2 : null,
            },
        },
    });
    const target = {
        provider: 'MC',
        replicas: flags.replicas && flags.replicas > 0 ?
            flags.replicas :
            1,
        targetId: args.deploymentTargetId,
        deploymentSettings: getDeploymentSettings(args, flags),
    };
    return { application: appData, target: target, name: args.name, labels: [] };
};
exports.handleCommonCreate = handleCommonCreate;
function getDeployConfig(args, flags) {
    const values = {};
    let properties = {};
    if (_.has(flags, 'propertiesFile')) {
        properties = (0, exports.parseFile)(flags.propertiesFile, exports.parseProperties);
    }
    else if (_.has(flags, 'property')) {
        properties = (0, exports.parseProperties)(flags.property, {});
    }
    values[MULE_AGENT_PROP_SERVICE] = {
        applicationName: args.name,
        properties: properties,
        secureProperties: {},
    };
    let scopeConfig = [];
    if (_.has(flags, 'scopeLoggingConfigFile')) {
        scopeConfig = (0, exports.parseFile)(flags.scopeLoggingConfigFile, parseScopeLoggingConfigs);
    }
    if (_.has(flags, 'scopeLoggingConfig')) {
        const arr = flags.scopeLoggingConfig.split(',');
        for (const x of arr) {
            const z = x.split(':');
            scopeConfig.push({ scope: z[0], logLevel: z[1] });
        }
    }
    values['mule.agent.logging.service'] = {
        scopeLoggingConfigurations: scopeConfig,
    };
    return values;
}
function parseScopeLoggingConfigs(propStrings) {
    const configs = [];
    if (!_.isArray(propStrings)) {
        propStrings = [propStrings];
    }
    _.each(propStrings, function (p) {
        configs.push(splitConfigArg(p));
    });
    return configs;
}
const configValidFormat = '{scope: scopeName, logLevel: logLevelType}';
function splitConfigArg(p) {
    const scopeString = 'scope';
    const logLevelString = 'logLevel';
    if (p[0] !== '{' && p[-1] !== '}') {
        throw new Error((0, util_1.format)('Invalid config format, expected %s  but was %s', configValidFormat, p));
    }
    const cfgs = p.slice(1, -1).split(/[,:=]+/);
    if (cfgs.length !== 4 || !cfgs[0].includes(scopeString) || !cfgs[2].includes(logLevelString)) {
        throw new Error((0, util_1.format)('Invalid config format, expected %s  but was %s', configValidFormat, p));
    }
    return { [scopeString]: cfgs[1].replace(/\s/g, ''), [logLevelString]: cfgs[3].replace(/\s/g, '') };
}
function getDeploymentSettings(args, flags) {
    const clustered = flags.clustered && flags.replicas && flags.replicas > 1 ?
        flags.clustered :
        false;
    if (flags.clustered && !clustered) {
        throw new Error('Requires at least 2 replicas to run in cluster mode');
    }
    return {
        clustered: clustered,
        disableAmLogForwarding: flags.disableAmLogForwarding ?
            flags.disableAmLogForwarding :
            false,
        enforceDeployingReplicasAcrossNodes: false,
        jvm: {},
        runtimeVersion: args.runtimeVersion,
        updateStrategy: flags.updateStrategy ?
            flags.updateStrategy :
            'rolling',
        http: {
            inbound: {
                publicUrl: flags.publicEndpoints ?
                    flags.publicEndpoints :
                    null,
                pathRewrite: flags.pathRewrite ? flags.pathRewrite : null,
                lastMileSecurity: flags.lastMileSecurity ?
                    flags.lastMileSecurity :
                    false,
                forwardSslSession: flags.forwardSslSession ?
                    flags.forwardSslSession :
                    false,
            },
        },
    };
}
